#ifndef UNKNOWNIMPL_H
#define UNKNOWNIMPL_H

#include "autoval.h"

template<class T>
class InterlockedSimpleOperators
{
  public:
    static inline T inc(T& t)
    {
      return InterlockedIncrement(&t);
    }


    static inline T dec(T& t)
    {
      return InterlockedDecrement(&t);
    }
};

class DefaultModuleControl
{
  public:
    inline static void LockModule(){}
    inline static void UnlockModule(){}

    inline static void LockServer(){}
    inline static void UnlockServer(){}
};



template <
  class I, 
  const CLSID* pcid=&IID_IUnknown, 
  class ModuleControl=DefaultModuleControl
>
class CUnknownImpl : public I
{

  //         
  // 

  template <
    class ModuleControl/*=DefaultModuleControl*/
  >

  class auto_module_lock
  {
    public:
      auto_module_lock()
      {
        ModuleControl::LockModule();
      }

      ~auto_module_lock()
      {
        //     
        // ModuleControl::Unlock() ,     Release

      }

  };


  private:
    auto_value<long,1,InterlockedSimpleOperators<long> > m_lRef;
    auto_module_lock<ModuleControl> _mlc_;
  public:
    virtual HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void** ppv)
    {
      if(IsEqualIID(riid,IID_IUnknown)){
        *ppv = dynamic_cast<IUnknown*>(dynamic_cast<I*>(this));
        AddRef();
        return S_OK;
      }
      if(IsEqualIID(riid,*pcid)){
        I* pI=dynamic_cast<I*>(this);
        *ppv = pI;
        pI->AddRef();
        return S_OK;

      }
      return E_NOINTERFACE;
    }

    virtual ULONG   STDMETHODCALLTYPE AddRef(void)
    {
      ++m_lRef;
      ModuleControl::LockModule();
      return m_lRef;
    }

    virtual ULONG   STDMETHODCALLTYPE Release(void)
    {
      long lTempRef=--m_lRef; 
      ModuleControl::UnlockModule();
      if(lTempRef==0){
        delete this;
        return 0;
      }
      else
        return lTempRef;
    }
    virtual ~CUnknownImpl()
    {
    }
};


template <
  class I, 
  class I2,
  const CLSID* pcid=&IID_IUnknown, 
  const CLSID* pcid2=&IID_IUnknown, 
  class ModuleControl=DefaultModuleControl
>

class CBiUnknownImpl : public CUnknownImpl<I,pcid,ModuleControl>, public virtual I2 
{

  public:
    virtual HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void** ppv)
    {
      if(IsEqualIID(riid,*pcid2)){
        *ppv = dynamic_cast<I2*>(this);
        AddRef();
        return S_OK;
      }
      return CUnknownImpl<I,pcid,ModuleControl>::QueryInterface(riid, ppv);
    }

    virtual ULONG   STDMETHODCALLTYPE AddRef(void)
    {
      return CUnknownImpl<I,pcid,ModuleControl>::AddRef();
    }

    virtual ULONG   STDMETHODCALLTYPE Release(void)
    {
      return CUnknownImpl<I,pcid,ModuleControl>::Release();
    }
    virtual ~CBiUnknownImpl()
    {
    }
};


template <
  class I, 
  class I2,
  class I3,
  const CLSID* pcid=&IID_IUnknown, 
  const CLSID* pcid2=&IID_IUnknown, 
  const CLSID* pcid3=&IID_IUnknown, 
  class ModuleControl=DefaultModuleControl
>

class CTriUnknownImpl : public CBiUnknownImpl<I,I2,pcid,pcid2,ModuleControl>, public virtual I3 
{

  public:
    virtual HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void** ppv)
    {
      if(IsEqualIID(riid,*pcid3)){
        *ppv = dynamic_cast<I3*>(this);
        AddRef();
        return S_OK;
      }
      return CBiUnknownImpl<I,I2,pcid,pcid2,ModuleControl>::QueryInterface(riid, ppv);
    }

    virtual ULONG   STDMETHODCALLTYPE AddRef(void)
    {
      return CBiUnknownImpl<I,I2,pcid,pcid2,ModuleControl>::AddRef();
    }

    virtual ULONG   STDMETHODCALLTYPE Release(void)
    {
      return CBiUnknownImpl<I,I2,pcid,pcid2,ModuleControl>::Release();
    }
    virtual ~CTriUnknownImpl()
    {
    }
};



#endif