// IPPageImpl.h: interface for the IPPageImpl class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_IPPAGEIMPL_H__8EDBC203_14ED_4D34_A3A4_241B4AF2E937__INCLUDED_)
#define AFX_IPPAGEIMPL_H__8EDBC203_14ED_4D34_A3A4_241B4AF2E937__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <atlbase.h>

template <class T>
class IPPageImpl : public IPropertyPage  
{
protected:
	BOOL m_bDirty;
	int m_lRef;

public:
	IPPageImpl()
	{
		T* pT = static_cast<T*>(this);
		pT->m_bDirty = FALSE;
		pT->m_lRef = 0;
	}

	~IPPageImpl()
	{
	}

	STDMETHOD(SetPageSite)(IPropertyPageSite *pPageSite)
	{
		return S_OK;
	}

	STDMETHOD(Activate)(HWND hWndParent, LPCRECT pRect, BOOL /* bModal */)
	{
		AFX_MANAGE_STATE(AfxGetStaticModuleState());

		T* pT = static_cast<T*>(this);

		//    ,      (    )
		//   ,   
		if (::IsWindow(pT->m_hWnd)) {
			pT->InitFields();
		}
		else {
			if (pRect == NULL)
				return E_POINTER;

			BOOL bval = pT->Create(pT->IDD, CWnd::FromHandle(hWndParent));

			CRect rc = *pRect;
			if ((rc.Width() <= 0) || (rc.Height() <= 0))
			{
				CRect rc2; pT->GetClientRect(&rc2);
				rc.right = rc.left + rc2.Width();
				rc.bottom = rc.top + rc2.Height();
			}
			Move(&rc);
			pT->ShowWindow(SW_NORMAL);
		}

		return S_OK;
	}

	STDMETHOD(Deactivate)( void)
	{
		AFX_MANAGE_STATE(AfxGetStaticModuleState());

		T* pT = static_cast<T*>(this);

		if (pT->m_hWnd)
		{
			if (::IsWindow(pT->m_hWnd))
				pT->DestroyWindow();
			pT->m_hWnd = NULL;
		}

		return S_OK;
	}

	STDMETHOD(Show)(UINT nCmdShow)
	{
		T* pT = static_cast<T*>(this);
		if (pT->m_hWnd == NULL)
			return E_UNEXPECTED;

		ShowWindow(pT->m_hWnd, nCmdShow);
		return S_OK;
	}

	STDMETHOD(Move)(LPCRECT pRect)
	{
		T* pT = static_cast<T*>(this);

		if (pT->m_hWnd == NULL)
			return E_UNEXPECTED;

		if (pRect == NULL)
			return E_POINTER;

		MoveWindow(pT->m_hWnd, pRect->left, pRect->top, pRect->right - pRect->left, pRect->bottom - pRect->top, TRUE);

		return S_OK;

	}

	//     
	STDMETHOD(Apply)(void)
	{
		AFX_MANAGE_STATE(AfxGetStaticModuleState());

		T* pT = static_cast<T*>(this);
		pT->OnApply();

		return S_OK;
	}

	void SetDirty(BOOL bDirty)
	{
		T* pT = static_cast<T*>(this);
		if (pT->m_bDirty != bDirty)
		{
			pT->m_bDirty = bDirty;
			// pT->m_pPageSite->OnStatusChange(bDirty ? PROPPAGESTATUS_DIRTY | PROPPAGESTATUS_VALIDATE : 0);
		}
	}

	STDMETHOD(IsPageDirty)(void)
	{
		T* pT = static_cast<T*>(this);
		return pT->m_bDirty ? S_OK : S_FALSE;
	}

	STDMETHOD(GetPageInfo)(PROPPAGEINFO *pPageInfo)
	{
		if (!pPageInfo)
			return E_FAIL;

		T* pT = static_cast<T*>(this);

		CString title = pT->GetTitle();
		if (title.GetLength())
		{
			USES_CONVERSION;
			pPageInfo->pszTitle = (LPOLESTR) CoTaskMemAlloc((title.GetLength() + 1) * 2);
			wcscpy(pPageInfo->pszTitle, A2W((LPCTSTR) title));
		}

		if (::IsWindow(pT->m_hWnd))
		{
			CRect rc; pT->GetClientRect(&rc);
			pPageInfo->size.cx = rc.Width();
			pPageInfo->size.cy = rc.Height();
		}

		return S_OK;
	}

	STDMETHOD(SetObjects)(ULONG nObjects, IUnknown **ppUnk)
	{
		ASSERT(FALSE);
		return E_NOTIMPL;
/*
		T* pT = static_cast<T*>(this);
		return pT->OnSetObjects(nObjects, ppUnk);
*/
	}

	STDMETHOD(Help)(LPCOLESTR pszHelpDir)
	{
		return S_OK;
	}

	STDMETHOD(TranslateAccelerator)(MSG *pMsg)
	{
		T* pT = static_cast<T*>(this);
		if ((pMsg->message < WM_KEYFIRST || pMsg->message > WM_KEYLAST) &&
			(pMsg->message < WM_MOUSEFIRST || pMsg->message > WM_MOUSELAST))
			return S_FALSE;

		return (IsDialogMessage(pT->m_hWnd, pMsg)) ? S_OK : S_FALSE;
	}

	// IUnknown
	HRESULT STDMETHODCALLTYPE QueryInterface(REFIID iid, void ** ppvObject)
	{
		if (iid == IID_IPropertyPage)
		{
			*ppvObject = dynamic_cast<IPropertyPage*>(this);
			return S_OK;
		}
		else
			return E_FAIL;
	}
	ULONG STDMETHODCALLTYPE AddRef(void)
	{
		T* pT = static_cast<T*>(this);
		pT->m_lRef++;
		return pT->m_lRef;
	}
	ULONG STDMETHODCALLTYPE Release(void)
	{
		T* pT = static_cast<T*>(this);
		pT->m_lRef--;
		if (pT->m_lRef <= 0)
		{
			delete pT;
			return 0;
		}
		return pT->m_lRef;
	}
};

#endif // !defined(AFX_IPPAGEIMPL_H__8EDBC203_14ED_4D34_A3A4_241B4AF2E937__INCLUDED_)
