#ifndef IDEVCTRL_H
#define IDEVCTRL_H
#include "unknwn.h"
#include "imodule.h"
#include "idevice.h"
#include "ipidctrl.h"


const int DEVCTRL_ERRORCODE_SUCCESS   = 0x0000;

const int DEVCTRL_TYPE_SIGNAL_CODE    = 0x0000;
const int DEVCTRL_TYPE_SIGNAL_ELECTR  = 0x0001;


typedef ULONG DEVCTRL_RESULT;
typedef ULONG DEVCTRL_DEVICE_ID;
typedef ULONG DEVCTRL_TYPE;

/// @cond INTERNAL
#define DEVCTRL_MAX_DEVICE 85

//Wrapper on devapi type TBusType
enum DEVCTRL_BUSTYPE
{
	DEVCTRL_MC_BUS = 1,
	DEVCTRL_EPP_BUS = 2,
	DEVCTRL_ISA_BUS = 3,
	DEVCTRL_PCI_BUS = 4,
	DEVCTRL_USB_BUS = 5,
	DEVCTRL_COMM_BUS = 6,
	DEVCTRL_PCMCIA_BUS = 7,
	DEVCTRL_ETHERNET_BUS = 9,
	DEVCTRL_TEDS_BUS = 10
};


typedef struct tagDEVCTRL_MC_CRATE_SLOT
{
	DEVCTRL_DEVICE_ID Crate;
	WORD SlotNo;
}DEVCTRL_MC_CRATE_SLOT;

typedef struct tagDEVCTRL_ME_SPI_SLOT 
{
	WORD CCType;
	WORD CCSN;
}DEVCTRL_ME_SPI_SLOT;


typedef struct tagDEVCTRL_PCI_SLOT
{
  DWORD dwBus;
  DWORD dwSlot;
  DWORD dwFunction;
}DEVCTRL_PCI_SLOT;

typedef struct tagDEVCTRL_ISA_SLOT
{
	WORD IOPort;
	char IRQLine;
}DEVCTRL_ISA_SLOT;		//    -

typedef struct tagDEVCTRL_EPP_SLOT
{
	WORD IOPort;
	char IRQLine;
}DEVCTRL_EPP_SLOT;		//    -

typedef struct tagDEVCTRL_USB_SLOT
{
	DWORD UniqueId;
	char SerialNumber[16];
}DEVCTRL_USB_SLOT;		//    -

typedef struct tagDEVCTRL_COMM_SLOT
{
	DWORD CommId;
}DEVCTRL_COMM_SLOT;	//    -

typedef struct tagDEVCTRL_PCMCIA_SLOT
{
	unsigned char uSocket;
	unsigned char uFunction;
}DEVCTRL_PCMCIA_SLOT;	//      -

typedef struct {
	DWORD port;
	char address[16];
	WORD CCType;
	WORD CCSN;
} DEVCTRL_ETHERNET_SLOT;		//    -

typedef struct
{
	char ID[8];
	WORD SN;
	WORD Type;
	WORD ChanNo;
} DEVCTRL_TEDS_OWNER;


#define MAX_TEDS_PINS 10
typedef struct
{
	DEVCTRL_TEDS_OWNER Pins[MAX_TEDS_PINS];
} DEVCTRL_TEDS_SLOT;

//Wrapper on devapi type TDeviceRoute 
//TDeviceRoute        // Full Address
typedef struct tagDEVCTRL_DEVICEROUTE{
	DEVCTRL_BUSTYPE BusType;	// PCI, ISA, EPP, USB, MC-bus ...
	union {
		DEVCTRL_PCI_SLOT PciSlot;	//    -
		DEVCTRL_ISA_SLOT IsaSlot;		//   -
		DEVCTRL_EPP_SLOT EppSlot;		// -
		DEVCTRL_USB_SLOT UsbSlot;		// USB-
		DEVCTRL_COMM_SLOT CommSlot;		// COMM-
		DEVCTRL_PCMCIA_SLOT PcmciaSlot;	// PCMCIA-
		DEVCTRL_MC_CRATE_SLOT CrateSlot;// Crate Slot
		DEVCTRL_ME_SPI_SLOT MeSpiSlot;  //   SPI-  (ME908, ME908-1)
		DEVCTRL_ETHERNET_SLOT EthernetSlot;		// Ethernet-
		DEVCTRL_TEDS_SLOT TEDS;
	} Location;
}DEVCTRL_DEVICEROUTE;
/// @endcond


/// Wrapper on devapi type TDevice
typedef struct tagDEVCTRL_DEVICE
{
/// @cond INTERNAL
	DEVCTRL_TYPE        DeviceType;
	DEVCTRL_TYPE        ObjectType;          ///< Type of object
	DEVCTRL_DEVICEROUTE Route;               ///< Full Address
	char                pchDeviceName[32];   ///< Name string; e.g:"M2408"
	char                pchRevString[32];    ///< Revision string; e.g:"Ver 4.5"
	char                pchDescription[100]; ///< Device description, Litera; e.g:"4 ,   
	                                         ///<     ."
	WORD                wRevisionNo;
	WORD                wSerialNo;
	char                pchDllName[100];      // Library Name
	void*               pCreator;
	void*               pFinder;
	WORD                HWProtocol;        // HardWare interface
/// @endcond
}DEVCTRL_DEVICE;



/// Wrapper on devapi type TDeviceEnum
typedef struct tagDEVCTRL_DEVICEENUM
{
	int             nFoundDevs;			            ///< Number of found devices
	DEVCTRL_DEVICE  DevInfo[DEVCTRL_MAX_DEVICE];	///< Devices info

} DEVCTRL_DEVICEENUM;


/// @interface IDeviceControl
///  ,   .      
/// .       
/// DevAPI.

class IDeviceControl: public IUnknown
{
public:
	///   DEVCTRL_DEVICE
	virtual HRESULT STDMETHODCALLTYPE InitDeviceRecord(DEVCTRL_DEVICE* pDevice) = 0;
	
	///   DEVCTRL_DEVICEENUM
	virtual HRESULT STDMETHODCALLTYPE InitDeviceEnumRecord(DEVCTRL_DEVICEENUM* pDeviceEnum) = 0;
	
	///    .    .
	virtual HRESULT STDMETHODCALLTYPE EditObject(IUnknown* pObject, HWND hParent) = 0;

	///  ()      DEVCTRL_DEVICE 
	///   DEVCTRL_TYPE
	virtual HRESULT STDMETHODCALLTYPE GetDeviceInfo(
			DEVCTRL_TYPE dtDeviceType, 
			DEVCTRL_DEVICE* pDeviceInfo
		) = 0;
	
	///        : DEVCTRL_DEVICE
	virtual HRESULT STDMETHODCALLTYPE CreateDeviceMain(
			DEVCTRL_DEVICE* pDeviceInfo, 
			IDevice** ppDevice
		) = 0;

	///    
	virtual HRESULT STDMETHODCALLTYPE GetAllDevices(DEVCTRL_DEVICEENUM* pDevices) = 0;
	
	///       
	virtual HRESULT STDMETHODCALLTYPE GetAllSearchDevices(
			DEVCTRL_TYPE dtDeviceType,
			DEVCTRL_DEVICEENUM* pDevices
		) = 0;


	///    
	virtual HRESULT STDMETHODCALLTYPE GetModuleInfo(
			DEVCTRL_TYPE dtDeviceType,
			DEVCTRL_TYPE dtModuleType, 
			DEVCTRL_DEVICE* pModuleInfo
		) = 0;
		
		
	///      
	virtual HRESULT STDMETHODCALLTYPE CreateDeviceModule(
			DEVCTRL_DEVICE* pModuleInfo, 
			IModule** ppModule
		) = 0;


	///      
	virtual HRESULT STDMETHODCALLTYPE AutoSearchModule(
			IDevice* pDevice, 
			DEVCTRL_DEVICEENUM* pDevEnum
		) = 0;

		
	///         
	virtual HRESULT STDMETHODCALLTYPE GetAllModules(
			DEVCTRL_TYPE dtDeviceType, 
			DEVCTRL_DEVICEENUM* pDevEnum
		) = 0;

	///     pid    .
	virtual HRESULT STDMETHODCALLTYPE GetPidControl(IPidControl** ppPC, IDevice* pDevice) = 0;

	///   .      
	/// 
	virtual HRESULT STDMETHODCALLTYPE StartDevice(IDevice* pDevice) = 0;

	///   .      
	/// 
	virtual HRESULT STDMETHODCALLTYPE StopDevice(IDevice* pDevice) = 0;
	
	
	///         DEVCTRL_DEVICE
	virtual HRESULT STDMETHODCALLTYPE GetDeviceByDeviceInfo(
			DEVCTRL_DEVICE* pDeviceInfo, 
			IDevice** ppDevice
		) = 0;
		

	///    
	virtual HRESULT STDMETHODCALLTYPE SearchDevices(DEVCTRL_DEVICEENUM* pDevices) = 0;
	
	///     .
	/// (   ,  DevAPI )
	virtual HRESULT STDMETHODCALLTYPE SystemConfigChanged(BOOL bChanged) = 0;


	///     . 
	///   :
	/// <br> IMnsSettingsControl
	/// <br> IScalarModulesCtrl
	/// <br> CONTROLLER_GUID_MIC_018
	/// <br>     
	virtual HRESULT STDMETHODCALLTYPE GetSpecDeviceController(
			IDevice* pDevice, 
			const GUID& guidControllerID, 
			IUnknown** ppController
		) = 0;

	///     ,    
	virtual HRESULT STDMETHODCALLTYPE RetrieveSettings(IUnknown* pObject, ULONG nPropID) = 0;
	
	///     
	virtual HRESULT STDMETHODCALLTYPE BurnSettings(IUnknown* pObject, ULONG nPropID) = 0;

	///   
	virtual HRESULT STDMETHODCALLTYPE ResetDevice(IUnknown* a_pObject) = 0;
	
};
#endif  