#ifndef IBUS_H
#define IBUS_H

#include "mebase.h"
#include "mestring.h"
#include "metypes.h"
#include "IDeviceInfo.h"
#include "IDevice.h"
#include "IBusLink.h"
#include "ISearhDevicesCallback.h"
#include "IValueList.h"


namespace mebius_daq{
//using namespace mebase;

class IDevice;
class ISearhDevicesCallback;
class IPipeOut;
class IDeviceInfo;

using mebase::system::mestring;

enum {
	BLSTFLAG_SKIP_NULL      = 0x0000,
	BLSTFLAG_SKIP_CONNECTED = 0x0001
};

// @interface IBus
///   
class IBus : public IObject
{
public:

	virtual MEBRESULT MEBCALLTYPE Init() = 0; 

	virtual MEBRESULT MEBCALLTYPE Lock() = 0;
	
	virtual MEBRESULT MEBCALLTYPE Unlock()=0;	

	///      
	virtual MEBRESULT MEBCALLTYPE GetHostDevice(
			IDevice** ppDevice ///< [out]   
		) = 0;

	///    
	virtual MEBRESULT MEBCALLTYPE GetID(
			mebase::BUSID* pID ///<[out]  
		) = 0;
	
	virtual MEBRESULT MEBCALLTYPE GetName(mestring& bus_name)=0;
	
	///    .
	///     ,    .
	virtual MEBRESULT MEBCALLTYPE SearchDevices(
			bool fAsync = true,                   ///< [in]     . true -  
			ISearhDevicesCallback* pCallBack = 0  ///< [in]   
		) = 0;

	///     
	virtual MEBRESULT MEBCALLTYPE WaitSearchDevices()=0;

	///   
	virtual MEBRESULT MEBCALLTYPE CancelSearchDevices()=0;

	///    .
	///        
	///       WaitSearchDevices.
	virtual MEBRESULT MEBCALLTYPE GetFoundDevicesCount(
			mebase::ULONG* pnCount ///< [out]  
		)=0;
	
	///       
	///        
	///       WaitSearchDevices.
	virtual MEBRESULT MEBCALLTYPE GetFoundDevice(
			mebase::ULONG nIndex,   ///< [in]  ,   0
			IDeviceInfo** ppInfo ///< [out]   
		)=0;


	///      .
	virtual MEBRESULT MEBCALLTYPE GetDevicesCount(
			mebase::ULONG* pnCount
		)=0;
	
	///     
	virtual MEBRESULT MEBCALLTYPE GetDevice(
			mebase::ULONG nIndex, ///< [in]  ,   0
			IDevice** ppDevice ///< [out]   
		)=0;


	///    
	virtual MEBRESULT MEBCALLTYPE LinkDevice(IDevice* pDevice) = 0;

	///    
	virtual MEBRESULT MEBCALLTYPE UnlinkDevice(IDevice* pDevice)=0;


	virtual MEBRESULT MEBCALLTYPE IoControl(
			mebase::ULONG nIoControlCode,
			const void* pInBuffer,
			mebase::ULONG nInBufferSize,
			void* pOutBuffer,
			mebase::ULONG nOutBufferSize,
			mebase::ULONG* pnBytesReturned,
			void* reserved=0
		) = 0;

	virtual MEBRESULT MEBCALLTYPE IoControlEx(
		mebase::TASKID nReceiverId,  //< id   
		mebase::ULONG nIoControlCode,
		const void* pInBuffer,
		mebase::ULONG nInBufferSize,
		void* pOutBuffer,
		mebase::ULONG nOutBufferSize,
		mebase::ULONG* pnBytesReturned,
		void* reserved = 0
		) = 0;


	///      
	virtual MEBRESULT MEBCALLTYPE GetAddressList(IValueList** ppAddressList, mebase::ULONG nFlags = 0) = 0;

};
}
#endif
	
