#ifndef ICHANNEL_H
#define ICHANNEL_H

#include <mebase.h>
#include <mestring.h>

#include "IObject.h"
#include "IValueList.h"
#include "IChangeChannelStateCallback.h"

using mebase::system::mestring;


///  -  
namespace mebius_daq{
//using namespace mebase;

///  
typedef enum {
	/// 
	CHTYPE_INPUT     = 0x0001,
	
	/// 
	CHTYPE_OUTPUT    = 0x0002,
	
	//    
	CHTYPE_IRREGULAR = 0x0004,

	/// 
	CHTYPE_STREAM    = 0x0010,
	
	/// 
	CHTYPE_SCALAR    = 0x0020,
	
	/// 
	CHTYPE_ANALOG    = 0x0100,
	
	/// 
	CHTYPE_DIGITAL   = 0x0200,
	
	// 
	CHTYPE_AUX       = 0x0400,
	
	// 
	CHTYPE_UTIME     = 0x0800
	
}CHANNEL_TYPE;

typedef enum {
	CHSTATE_CONNECTED  = 0x0001,
	CHSTATE_PROGRAMMED = 0x0002,
	CHSTATE_DELETED    = 0xFFFF

}CHANEL_STATE;


class IDataPipeOut;
class IDataPipeIn;
class IDevice;
class IValueList;
class IChangeChannelStateCallback;

/// @interface IChannel
///  .<br>
///    ,   
/// <*>   .
/// <*>    ()
/// <*>   .
/// <br>
///             
///      . <br>
///     ,    
///
///       


class IChannel : public mebius_daq::IObject
{
public:
	///  .
	virtual MEBRESULT MEBCALLTYPE Init() = 0;
	
	///  .
	virtual MEBRESULT MEBCALLTYPE Uninit() = 0;	
	
	///  
	virtual MEBRESULT MEBCALLTYPE Destroy() = 0;
	
	///     
	virtual MEBRESULT MEBCALLTYPE Connect() = 0;
	
	///     
	virtual MEBRESULT MEBCALLTYPE Disconnect() = 0;
	
	
	///  
	///    <br>
	
	virtual MEBRESULT MEBCALLTYPE GetState(
			mebase::ULONG* pState
		) = 0;
	
	///   
	virtual MEBRESULT MEBCALLTYPE GetAddress(
			mestring& msAddress         ///<[out]   
		) = 0;

	///     
	virtual MEBRESULT MEBCALLTYPE GetType(
			mebase::ULONG* pnType       ///<[out]   
		) = 0;

	///     
	virtual mebase::ULONG MEBCALLTYPE GetNumber() const = 0;
	
	///     
	virtual MEBRESULT MEBCALLTYPE GetDevice(IDevice** ppDevice) const = 0;
	
	
	///         
	virtual mebase::ULONG MEBCALLTYPE Connected() const = 0;

	
	///   
	virtual MEBRESULT MEBCALLTYPE GetFreq(float* pdblFreq) const = 0;

	///   
	virtual MEBRESULT MEBCALLTYPE SetFreq(float dblFreq) = 0;
	
	///     
	virtual MEBRESULT MEBCALLTYPE GetFreqList(IValueList** ppList) = 0;

	///    
	virtual MEBRESULT MEBCALLTYPE LinkOutputStream(
			IDataPipeOut* pStream,      ///<[in]    
			mebase::ULONG nReserved     ///<[in] Reserved,        - 
		) = 0;
		
	///    
	virtual MEBRESULT MEBCALLTYPE LinkInputStream(
			IDataPipeIn* pStream,       ///<[in]    
			mebase::ULONG nReserved     ///<[in] Reserved,        - 
		) = 0;

	///     
	virtual MEBRESULT MEBCALLTYPE ZBalance(mebase::ULONG* a_pVal) = 0;
	
	///       
	virtual MEBRESULT MEBCALLTYPE SubscribeEvents(IChangeChannelStateCallback* a_pCallback) = 0;

	///       
	virtual MEBRESULT MEBCALLTYPE UnSubscribeEvents(IChangeChannelStateCallback* a_pCallback) = 0;

	
};
}
#endif
