#ifndef IDEVICE_H_MEB
#define IDEVICE_H_MEB

#include <mestring.h>
#include "mebase.h"
#include "MeVariant.h"
#include "IObject.h"
#include "IDeviceInfo.h"
#include "IBus.h"
#include "IPipes.h"
#include "IScansList.h"
#include "IChansList.h"
#include "IBusList.h"
#include "IPropStorage.h"
#include "IChangeDeviceStateCallback.h"
#include "IZBalanceCallback.h"

namespace mebius_daq{

///     GetChannels 
enum{
	GCHLF_CONNECTED    = 0x0001,
	GCHLF_DISCONNECTED = 0x0002,
	GCHLF_ALL          = GCHLF_CONNECTED|GCHLF_DISCONNECTED
};


const bool ASYNC_OPERATION=true;

class ITrigger;
class IDeviceInfo;
class IPipeOut;
class IBusList;
class IBus;


/// @interface IDevice
class IDevice : public IObject
{
public:
	
	///  
	virtual MEBRESULT MEBCALLTYPE MEBCALLTYPE Init(
			IChangeDeviceStateCallback* pChangeDeviceStateCallback = 0 ///<[in]     
		) = 0;	
	
	/// 
	virtual MEBRESULT MEBCALLTYPE Uninit() = 0;

	///      \n
	/// ,    ip  ethernet 
	virtual MEBRESULT MEBCALLTYPE Attach(IDeviceInfo * const pLink) = 0;	
	
	///      
	virtual MEBRESULT MEBCALLTYPE Unattach() = 0;	

	///      
	virtual MEBRESULT MEBCALLTYPE Connect(
			bool fAsync=false ///< [in]     . true -  
		)=0;

	///    
	virtual MEBRESULT MEBCALLTYPE Disconnect()=0;
	
	///     
	virtual MEBRESULT MEBCALLTYPE WaitConnecting()=0;

	///  . \n
	///     .
	virtual MEBRESULT MEBCALLTYPE Program(
			bool fAsync=true ///< [in]     . true -  
		)=0;
	
	
	///     
	virtual MEBRESULT MEBCALLTYPE WaitProgramming()=0;

	/// @todo 
	///   
	///    
	///     Connected
	///	virtual MEBRESULT MEBCALLTYPE Reset() = 0;




	///       . /n
	///        
	virtual MEBRESULT MEBCALLTYPE Start(
			ITrigger* pTrigger=0 ///< [in]   .    0, 
													 ///<    
		)=0;

	///  . \n
	///     .
	virtual MEBRESULT MEBCALLTYPE Stop(
			bool fAsync=false ///< [in]     . true -  
		)=0;
	
	///     
	virtual MEBRESULT MEBCALLTYPE WaitStopping()=0;


	///    
	/// @todo    
	virtual MEBRESULT MEBCALLTYPE GetChannels(IChansList** ppChannels, const mebase::ULONG a_nFlags)=0;

	///     
	virtual MEBRESULT MEBCALLTYPE GetScans(IScansList** ppScans)=0;

	///   RAW    
	virtual MEBRESULT MEBCALLTYPE LinkOutputStream(
			IPipeOut* pStream,  ///<[in]    
			mebase::ULONG nReserved     ///<[in] Reserved,        - 
		)=0;
	
	///       
	virtual MEBRESULT MEBCALLTYPE GetInputStream(
			IPipeOut** ppStream,					///<[in]    
			mebase::ULONG nReserved=0     ///<[in] Reserved,        - 
		)=0;

	///   
	virtual MEBRESULT MEBCALLTYPE GetBusList(IBusList** ppBusList) = 0;

	///         
	///        BUSID_HOST ( BUSID_NULL,  0)
	virtual MEBRESULT MEBCALLTYPE GetBus(IBus** pBus,	const mebase::ULONG id) = 0;
																									 
	virtual MEBRESULT MEBCALLTYPE GetBusByName(IBus** pBus, mebase::system::mestring& bus_name)=0;
																									 
	//        XDevice
	//      ...
	///
	virtual MEBRESULT MEBCALLTYPE SetParent(IDevice* pDevice)=0;
	
	///
	virtual MEBRESULT MEBCALLTYPE LinkChild(IDevice* pDevice)=0;
	
	virtual MEBRESULT MEBCALLTYPE LinkChangeDeviceStateCallback(IChangeDeviceStateCallback* pCallback)=0;
	
	

	///   
	virtual MEBRESULT MEBCALLTYPE GetAddress(mestring& msAddress) = 0;

	virtual mebase::ULONG MEBCALLTYPE GetId() const = 0;	
/*
	///  , 0  ,  0 - 
	virtual mebase::ULONG MEBCALLTYPE CheckState(DEVICESTATE state) = 0;
*/
	///   
	virtual mebase::DEVICESTATE MEBCALLTYPE GetState() const = 0;


	///     
	virtual MEBRESULT MEBCALLTYPE StoreSettings(mebase::system::IPropStorageOut* pStorage)=0;

	///     
	virtual MEBRESULT MEBCALLTYPE RestoreSettings(mebase::system::IPropStorageIn* pStorage)=0;

	virtual MEBRESULT MEBCALLTYPE CallCommand(mebase::ULONG a_cmd, mebase::BYTE* pBufIn=NULL, mebase::USHORT a_szIn = 0, mebase::BYTE* pBufOut=NULL, mebase::USHORT a_szOut=0) = 0;

	virtual MEBRESULT MEBCALLTYPE ZBalanceMulti(IChansList* chans, IValueList** vals, IZBalanceCallback* callback = 0) = 0; 
	
	///    
	virtual MEBRESULT MEBCALLTYPE ZBalanceMultiEx(
			IChansList* chans,
			IValueList** vals,
			IValueList** status,
			IZBalanceCallback* callback = 0,
			mebase::ULONG reserved = 0
		) = 0;

	///  IoControl   
	///     
	virtual MEBRESULT MEBCALLTYPE IoControl(
		mebase::ULONG nIoControlCode,
		const void* pInBuffer,
		mebase::ULONG nInBufferSize,
		void* pOutBuffer,
		mebase::ULONG nOutBufferSize,
		mebase::ULONG* pnBytesReturned,
		void* reserved = 0
		) = 0;

	virtual MEBRESULT MEBCALLTYPE IoControlEx(
		mebase::TASKID nReceiverId,  //< id   
		mebase::ULONG nIoControlCode,
		const void* pInBuffer,
		mebase::ULONG nInBufferSize,
		void* pOutBuffer,
		mebase::ULONG nOutBufferSize,
		mebase::ULONG* pnBytesReturned,
		void* reserved = 0
		) = 0;

	///      
	///           
	virtual MEBRESULT MEBCALLTYPE LinkInputDevice(
		IDevice* pDevice,  ///<[in]    
		mebase::ULONG nReserved     ///<[in] Reserved,        - 
		)=0;
	///      
	virtual MEBRESULT MEBCALLTYPE UnlinkInputDevice(
		IDevice* pDevice  ///<[in]    
		)=0;

	virtual MEBRESULT MEBCALLTYPE GetParent(IDevice** pDevice) const = 0;
};

}
#endif
