#ifndef MESTRING_H
#define MESTRING_H

#ifdef _MSC_VER
	#ifndef _CRT_SECURE_NO_WARNINGS
		#define _CRT_SECURE_NO_WARNINGS
	#endif
#endif



#include <string>
#include <sstream>
#include <assert.h>
#ifdef __BORLANDC__ 
#include <string.h>
#endif

#ifdef __clang__ 
#include <string.h>
#endif


namespace mebase{namespace system{

//! @class mestring 
//!         .
//!        std::string \n
//!    \n 
//!         \n
//!  <b>  </b>:\n
//! @code
//!  void foo(mestring s);
//!  . . .
//!  mestring a="Hello World!";
//!  foo(a);
//! @endcode
//!     foo :\n
//! @code
//! void foo(const mestring& s);
//! @endcode



#ifdef _MSC_VER
	#define MEX_CALLTYPE __stdcall
#else
	#ifdef __BORLANDC__
		#define MEX_CALLTYPE __stdcall
	#else 
		#define MEX_CALLTYPE 
	#endif
#endif


class i_string_impl
{
public:
	virtual void MEX_CALLTYPE destroy() = 0;
	virtual const char* MEX_CALLTYPE get() const = 0;
};

class string_impl: public i_string_impl
{
public:
	string_impl(const char* p)
	{
		sz_ = strlen(p);
		pch_ = new char[sz_ + 1];
		strncpy(pch_, p, sz_ + 1);
	}

	virtual const char* MEX_CALLTYPE get() const
	{
		return pch_;
	}
	virtual void MEX_CALLTYPE destroy()
	{
		delete this;
	}
	virtual ~string_impl()
	{
		delete[] pch_;
	}

private:
	char* pch_;
	int sz_;
friend class mestring;
};

class mestring
{

public:

	mestring(const std::string& s)
	{
		impl_=new string_impl(s.c_str());
	}

	explicit mestring(const char* s)
	{
		impl_=new string_impl(s);
	}
	mestring()
		:impl_(0)
	{}


	///     
	mestring(const mestring& r)
	{
		impl_=r.impl_;
		const_cast<mestring&>(r).impl_=0;
	}

	const mestring& operator = (const std::string& s)
	{
		if(impl_) {
			impl_->destroy();
		}
		impl_=new string_impl(s.c_str());
		return *this;
	}


	~mestring()
	{
		if(impl_) {
			impl_->destroy();
			impl_ = 0;
		}
	}

	operator const std::string ()const
	{
		assert(impl_);
		return std::string(impl_->get());
	}


protected:
	const mestring& operator = (const mestring& r)
	{
		if(impl_) {
			impl_->destroy();
		}
		impl_=r.impl_;
		const_cast<mestring&>(r).impl_=0;
		return *this;
	}


private:
	i_string_impl* impl_;
friend class CMeVariant;
};


// number to std::string
template <typename T>
static inline std::string NumberToString ( T Number, int Precision = 4 )
{
	std::ostringstream ss;
	ss.precision(Precision);
	ss << std::fixed << Number;
	return ss.str();
}

}}
#endif
