#ifndef IDATAFOLDERTEMPLATE_H
#define IDATAFOLDERTEMPLATE_H

#include "IRecorder.h"
#include "ISettingsStorage.h"


// {ECEBC50A-A304-4348-8AFB-52BBD5FC95CE}
static const GUID SEPARATOR_ITEM_ID = 
	{ 0xecebc50a, 0xa304, 0x4348, { 0x8a, 0xfb, 0x52, 0xbb, 0xd5, 0xfc, 0x95, 0xce } };



// {D972F15F-28A7-4aab-B380-EDF749891F31}
extern const GUID IID_IDataFolderTemplateItem;

/// @interface IDataFolderTemplateItem
class IDataFolderTemplateItem : public IUnknown
{
public:
	virtual HRESULT STDMETHODCALLTYPE Init(IRecorder* a_pRecorder) = 0;
	virtual HRESULT STDMETHODCALLTYPE Uninit()  = 0;

	///    ,  
	virtual HRESULT STDMETHODCALLTYPE GetItemCaption(BSTR* a_pbstrName) const = 0;
	///    
	virtual HRESULT STDMETHODCALLTYPE GetItemId(GUID* a_pId) const = 0;
	
	///         
	virtual HRESULT STDMETHODCALLTYPE GetText(BSTR* a_pbstrName) const = 0;

	///   
	/// :  v3.3.2.19   
	virtual HRESULT STDMETHODCALLTYPE OnStart(long a_fRecMode) = 0;

	///   
	virtual HRESULT STDMETHODCALLTYPE OnStop() = 0;

	///     
	virtual HRESULT STDMETHODCALLTYPE Edit(HWND a_hParent) = 0;

	///  
	virtual HRESULT STDMETHODCALLTYPE StoreSettings(ISettingsStorageOut* pStorage) const = 0;
	
	///  
	virtual HRESULT STDMETHODCALLTYPE RestoreSettings(ISettingsStorageIn* pStorage) = 0;


	
};

// {D972F15F-28A7-4aab-B380-EDF749891F31}
extern const GUID IID_IDataFolderTemplateItemFactory;

class IDataFolderTemplateItemFactory : public IUnknown
{
public:
	virtual HRESULT STDMETHODCALLTYPE GetClassName(BSTR* a_pbstrName) const = 0;
	virtual HRESULT STDMETHODCALLTYPE GetClassId(GUID* a_pId) const = 0;
	virtual HRESULT STDMETHODCALLTYPE CreateItem(IDataFolderTemplateItem** a_ppItem) = 0;
};

// {6EE063BE-28A9-4fe2-B557-9EC2F265E43E}
extern const GUID IID_IDataFolderTemplateItemsManager;

class IDataFolderTemplateItemsManager : public IUnknown
{
public:
	///   
	virtual HRESULT STDMETHODCALLTYPE RegisterItemClass(IDataFolderTemplateItemFactory* a_pClass) = 0;
	
	/// 
	virtual HRESULT STDMETHODCALLTYPE UnregisterItemClass(GUID* a_pId) = 0;
	
	///    / 
	virtual HRESULT STDMETHODCALLTYPE GetClassCount(ULONG* a_pnCount) const = 0;

	///  id  / 
	virtual HRESULT STDMETHODCALLTYPE GetClassId(ULONG a_nIndex, GUID* a_pId) const = 0;
	
	///    / 
	virtual HRESULT STDMETHODCALLTYPE GetClassName(ULONG a_nIndex, BSTR* a_pbstrName) const = 0;

	///   
	virtual HRESULT STDMETHODCALLTYPE CreateItem(GUID a_pId, IDataFolderTemplateItem** a_ppItem) = 0;
};

// {6061F6AB-6B6B-4f37-9C37-21FA45EFDD91}
extern const GUID IID_IDataFolderTemplate;

class IDataFolderTemplate : public IUnknown
{
public:
	///   
	virtual HRESULT STDMETHODCALLTYPE GetItemsCount(ULONG* a_pnCount) const = 0;
	virtual HRESULT STDMETHODCALLTYPE GetItem(ULONG a_nIndex, IDataFolderTemplateItem** a_ppItem) = 0;

	virtual HRESULT STDMETHODCALLTYPE AddItem(ULONG a_nIndex, IDataFolderTemplateItem* a_pItem) = 0;
	virtual HRESULT STDMETHODCALLTYPE DeleteItem(ULONG a_nIndex) = 0;
	virtual HRESULT STDMETHODCALLTYPE GeneratePath(BSTR* a_pbstrName) const = 0;

};

#endif