/******************************************************************************/
/*  Recorder\iface                                                            */
/*                                                                            */
/*    IRecorder                                             */
/*                                                  */
/*                                                                            */
/*                                                                            */
/*    : irecorder.h                                           */
/*  : MVC++6.0                                                      */
/*                                                                            */
/*  (c)  "" 2001, 2002 .                                             */
/******************************************************************************/




#ifndef IRECORDER_H
#define IRECORDER_H

#include "rcerrors.h"
#include "imodule.h"
#include "rcconsts.h"
#include "itag.h"
#include "idevice.h"
#include "propaccess.h"


typedef DWORD HARDLINK;
typedef __int64 TAGID;

///  recorder
/// @enum RECORDER_STATE
enum RECORDER_STATE 
{
	RS_STOP                    = 0x00000001, ///< 
	RS_VIEW                    = 0x00000002, ///<   /
	RS_REC                     = 0x00000004, ///<  
	RS_PLAYING                 = 0x00000008, ///<  
	RS_PLAY                    = 0x00000008, ///<  
	RS_BASESTATE               = RS_REC|RS_VIEW | RS_STOP|RS_PLAYING, ///<   
	RS_HARDWAREFAULT           = 0x00000010, ///<     
	RS_INITFAULT               = 0x00000020, ///<     
	RS_NEEDDEVICERESET         = 0x00000040, ///<  Reset 
	RS_NEEDHARDWARERESET       = 0x00000040, ///<  Reset 

	RS_NEEDSOFTWARERESET       = 0x00000080, ///<    
	RS_NEEDLINKSREFRESH        = 0x00000100, ///<   
	RS_FAULT                   = RS_HARDWAREFAULT | RS_INITFAULT,

	RS_PLAYMODE                = 0x00000200, ///<      
	RS_SIGNALLOADED            = 0x00000400, ///<   
	RS_CONFIGCHANGED           = 0x00000800, ///<  ,  

	RS_CONFIGMODE              = 0x00001000, ///<  
	RS_PAUSE                   = 0x00002000, ///<  

	RS_WAIT_REC_FINISH         = 0x00004000, ///<    
	RS_CALIBRATE_MODE          = 0x00008000, ///<  



	RS_PACKETLOST              = 0x00010000, ///<    
	RS_RECEIVEERROR            = 0x00020000, ///<   
	RS_PLAYMODE_ENABLED        = 0x00040000, ///<   
	RS_ZBALANCE_PROC           = 0x00080000, ///<     

	RS_ERROR_CONDITION         = 0x00100000, ///<    
	RS_WARNING_CONDITION       = 0x00200000, ///<    




	RS_ENABLEDPAUSE            = 0x40000000, 
	RS_TERMINATION             = 0x80000000, ///<   

	RS_FULLMASK                = 0xFFFFFFFF
};

///  
/// @enum RECORDER_NOTIFY
enum RECORDER_NOTIFY 
{
	RCN_RECONFIG,               ///< 
	RCN_VIEW,                   ///<   
	RCN_REC,                    ///<  
	RCN_STOP,                   ///<  
	RCN_SHOW,                   ///<    
	RCN_HIDE,                   ///<   
	RCN_CLOSE,                  ///<    
	RCN_SETCURFORM,             ///<  
	RCN_ENABLESETUP,            ///<  /  
	RCN_CLOSEPLUGIN,            ///<   
	RCN_SETCURTAG,              ///<      a_dwParam =  
	RCN_CHANGECURTAG,           ///<    =   + a_dwParam
	RCN_CHANGETAGRANGE,         ///<      use TAGRANGEDATA
	RCN_RESORTTAGS,             ///<  
	RCN_LAUNCHWINPOS,           ///<     
	RCN_TEST,                   ///<   
	RCN_SAVECONFIG,             ///<  
	RCN_CLOSEALLPLUGINS,        ///<    
	RCN_OPENPLUGIN,             ///<    
	RCN_REG_DF_CHANGER,         ///<  ,   
	RCN_PLAY,                   ///<   
	RCN_ON_PLAY_MODE,           ///<      
	RCN_ON_REC_MODE,            ///<      
	RCN_SUBSCRALARMSEVENT,      ///<      
	                            ///<    IAlarmEventHandler
	RCN_UNSUBSCRALARMSEVENT,    ///<      
	RCN_PLAY_SOUND_NOTIFY,      ///<   ,  @ref RCSNDNOTIFY
                                
	RCN_INTERNAL_BASE           = 0x8000,
	RCN_UPDATEGROUPSLINKS,      ///<      dwParam == ByNames:BOOL 
	RCN_DRIVERSRESET,           ///<    @ref RCNDRIVERSRESETFLAGS
	RCN_DRIVERSCONFIG,          ///<    @ref RCNDRIVERSCONFIGFLAGS
	RCN_CHECKTAGSNAMES,         ///<     
	RCN_BROADCAST,              ///<   
	RCN_SWITCH_CALIBR_MODE,     ///<    
                                
	RCN_DEACTIVATE_DEVICES,     ///<        
	                            ///<    TRUE/FALSE
	RCN_RUN_IN_RC_THREAD,       ///<         RCTRCOMMAND
	                            ///<  RCTRCOMMAND.pPlgDest==0   
	RCN_ACTION_POPUP_SETUP,     ///<         
	RCN_ACTION_POPUP_SEL_FRAME, ///<          

	RCN_ACTION_SAVE_CFG,        ///<        rc
	RCN_ACTION_SAVE_CFG_AS,     ///<        rc
	RCN_ACTION_LOAD_CFG,        ///<        rc
	RCN_DRIVER_RESET_EX,        ///<   ,     
	RCN_DRIVER_CONFIG_EX,       ///<   ,     
	RCN_MARK_DEVICE_FOR_RESET,  ///<  ,         
	RCN_RESET_CONFIG,           ///<  
	RCN_ACTION_CLOSE_APP,       ///<       
	RCN_FORCE_CLOSE,            ///<       
	RCN_RESET_DFT_INDEX         ///<         
	
};

/// 
enum RECORDER_BROADCAST 
{
	RCBC_PLUGINSLISTUPDATED
};


/// @struct RCSNDNOTIFY
/// @brief    @ref RCN_PLAY_SOUND_NOTIFY
typedef struct tagRCSNDNOTIFY
{
	ULONG cbSize;
	ULONG nType;
	union {
		char pchSoundFile[MAX_PATH+1];
		ULONG id;
	};

}RCSNDNOTIFY, *PRCSNDNOTIFY;


///   
enum {
	RCSNT_FROM_FILE = 0x0001
};

typedef struct tagRCTRCOMMAND
{
	ULONG cbSize;
	void* pPlgDest;
	ULONG nCommand;
	ULONG nData;
}RCTRCOMMAND, *PRCTRCOMMAND;


/// @enum SECURITYSTATE    @ref RCPROP_SECURITY_STATE
enum SECURITYSTATE{
	SECST_FULL_ACCESS = 0xFFFF,
	SECST_ALLOW_ENTER_CONFIG = 0x0001
};


///  
enum {
	RCPROP_HOSTDEVICE,                  ///< [in]    [out]  
	RCPROP_DATAFOLDER,                  ///<                                         [in] [out]
	RCPROP_STARTRECLEVCTRLPROPS,        ///<      RCLEVELCONTROLPROPS       [out]
	RCPROP_STARTTIME,                   ///<   / VT_FILETIME             [out]
	RCPROP_CALIBRDBNAME,                ///<                   [out] VT_BST  
	RCPROP_REFRESHPERIOD,               ///<  /           [in][out] VT_R8
	RCPROP_VIEWTIME,                    ///<  /         [in][out] VT_R8
	RCPROP_VERSION,                     ///<                                             [out] VT_I4
	RCPROP_TAGSSORTMODE,                ///<                                    [in][out] VT_I4
	RCPROP_TIMERTICPERIOD,              ///<    
	RCPROP_MODIFYFRAMENAME,             ///<      
	RCPROP_CONVERTTOUSML,               ///<      
	RCPROP_ENABLEDLEVELSTARTREC,        ///<     
	RCPROP_TRIGGERSTART,                ///<   
	RCPROP_ENABLEDAUTOSTOPTIME,         ///<     
	RCPROP_AUTOSTOPTIME,                ///<       
	RCPROP_CONFIGNAME,                  ///<   
	RCPROP_UISERVERLINK,                ///<   UI  
	RCPROP_SUMMARYDATAFLOW,             ///<   
	RCPROP_PROJECTNAME,                 ///<   (.cfg) [out] VT_BSTR
	RCPROP_CURRENTTAGNAME,              ///<    [in, out] VT_BSTR
	RCPROP_CHANGEDSETTINGS,             ///<   SCF_XXX [in, out] U32
	RCPROP_ENABLEDLEVELSTOPREC,         ///<     
	RCPROP_STOPRECLEVCTRLPROPS,         ///<      RCLEVELCONTROLPROPS        [out]
	RCPROP_PREHISTORY_ENABLED,          ///<                                         [in, out] VT_BOOL
	RCPROP_PREHISTORY_TIME,             ///<                                    [in, out] VT_I4
	RCPROP_RESET_TIME_AT_START,         ///<       / [in, out] VT_BOOL
	RCPROP_TRIGGER_START_MODE,          ///<     1 - "1", 0 - "0"
	RCPROP_CALIBRATOR,                  ///<     [out] VT_UNK
	RCPROP_ENABLE_SPACE_RESERVE,        ///<       [in, out] VT_BOOL
	                                    ///<  . " "
	RCPROP_SPACE_RESERVE_TIME,          ///<       [in, out] VT_R8
	RCPROP_SECURITY_STATE,              ///<    / [out]VT_UI4
	RCPROP_SYS_LOG,                     ///<    [out] VT_UNK
	RCPROP_PUT_CFG_FILE_TO_DATA,        ///<      
	                                    ///<    [in, out] VT_BOOL
	RCPROP_FRAME_NAME,                  ///<   ,   [in, out] VT_BSTR
	RCPROP_LAST_DATA_LOCATION,          ///<     [out] VT_BSTR
	RCPROP_USER_LOG,                    ///<  ()   [out]VT_UNK
	                                    ///<         
	                                    ///<        
	RCPROP_BATTERY_INFO,                ///<     [out] VT_BSTR
	RCPROP_MAIN_TAGS_TABLE,             ///<  	ImeTagsTable [out] VT_UNK
	RCPROP_FLUSH_BEFORE_CLOSE,          ///<       [in, out] VT_BOOL {v: 3.0.4.53}
	RCPROP_WRITE_CODES,                 ///<        
	                                    ///<  [out] VT_BOOL {v: 3.0.4.65, 3.0.5.2}
	RCPROP_ALARM_GISTER_BY_RANGE,       ///<     - 
	                                    ///<       [out] VT_BOOL {v: 3.0.6.15, 3.0.5.16}
	RCPROP_PREVIEW_ENABLED,             ///<   [out] VT_BOOL {v: 3.0.7.21}
	RCPROP_SAVE_CONFIG_DIALOG_AT_EXIT,  ///<      ,   
	                                    ///<     [out] VT_BOOL {v: 3.0.7.40} 
	RCPROP_VIEW_AT_STOP,                ///<          [in, out] VT_BOOL {v: 3.0.9.24} 
	RCPROP_DATAF_TEMPLATE_MANAGER,      ///<     [out] VT_UNK {v: 3.3.0.37
	RCPROP_DATAF_TEMPLATE_ENABLED,      ///<      [in, out] VT_BOOL {v: 3.3.0.39,


	RCPROP_INTERNAL_BASE                = 0x8000,
	RCPROP_USERMODE,                    ///<   Simple/Advanced/More                    [out] VT_I4
	RCPROP_SHOWDISBALANCE,              ///<                        [out] VT_BOOL
	RCPROP_ENABLEDRECPAUSE,             ///<      [out] VT_BOOL
	RCPROP_LOST_SRC_TG_CNT,             ///<      [out] VT_UI4
	RCPROP_PLAYMODE_ENABLED,            ///<     [in, out] VT_BOOL
	RCPROP_TIME,                        ///<   
	                                    ///<      ,
	                                    ///< ,   (    ) [in, out] VT_R8
	RCPROP_INCLUDE_DEV_NAME_IN_TAG,     ///<      
	                                    ///< - [out] VT_BOOL
	RCPROP_AUTO_STORE_VISUAL_CHANGES,   ///<       
	RCPROP_TIME_MASHINE_PROPS,          ///<      [in, out] IUnknown
	RCPROP_FLUSH_SCALAR_TAGS,           ///<    ()    [out] VT_BOOL
	RCPROP_FLUSH_SCALAR_PERIOD ,        ///<     ()    [out] VT_I4 []
	RCPROP_TIMESTAMP_AT_RECORD_START,   ///<      .mera    
	                                    ///<        .
	                                    ///<           [out] VT_BOOL

	RCPROP_USE_NEW_TRENDS,              ///<           [in,out] VT_BOOL
	RCPROP_WRITE_TARED,                 ///<             [in,out] VT_BOOL
	RCPROP_DATA_FOLDER_TEMPLATE,        ///<    [out] VT_UNK {v: 3.3.0.37,
	RCPROP_DATA_FRAME_TEMPLATE          ///<      [out] VT_UNK {v: 3.3.0.37,
};

///      RCPROP_TAGSSORTMODE           
/// @enum RCSORTMODE
enum RCSORTMODE{
	SRTMD_BY_NAME,                ///<  
	SRTMD_BY_ADDRESS,             ///<   
	SRTMD_BY_DEVICENAME           ///<    
};




///    DriverReset (@ref RCN_DRIVERSRESET)
enum RCNDRIVERSRESETFLAGS{
	DRF_SHOWPROCINDICATOR = 0x0001, ///<    
	DRF_ONLYHOST = 0x0002           ///<    
};

///    DriverConfig (@ref RCN_DRIVERSCONFIG)
enum RCNDRIVERSCONFIGFLAGS{
	DCF_SHOWPROCINDICATOR = 0x0001, ///<    
	DCF_ONLYHOST = 0x0002           ///<    
};


///    
typedef enum {LV_EQUAL, LV_ABOVE, LV_BELOW}LEVEL;

typedef struct tagRCLEVELCONTROLPROPS{
	ITag*     pTag;
	char      pchTagName[STDSTRSIZE];
	double    dblLevel;
	LEVEL     level;
}RCLEVELCONTROLPROPS,*PRCLEVELCONTROLPROPS;

///   
/// @ref RCN_CHANGETAGRANGE  -      
typedef struct tagTAGRANGEDATA{
	ITag*  pTag;
	double dblBX;
	double dblBY;
	double dblEX;
	double dblEY;
}TAGRANGEDATA, *PTAGRANGEDATA;



///     
/// Config mode sharing flags
enum {
	CMSF_CONFIGCHANGED = 0x0001     ///<     
};

///    
enum{
	IESF_DEFAULTSETTINGS = 0x0001,  ///<  / 
																//  
	IESF_LOCAL_MODE = 0x0002
};

///      
///     @ref PN_LEAVERCCONFIG
enum{
	SCF_NOTING      = 0x0000,
	SCF_TAGSLIST    = 0x0001,
	SCF_TAGSCONTENT = 0x0002
//  SCF_ALL         = 0x0FFFF
};


typedef DWORD RECORDERSTATE;
#include "itag.h"
class IVForm;



/// @interface IRecorder
class IRecorder : public IUnknown
{
public:
	///   
	virtual bool     STDMETHODCALLTYPE RegisterIForm(IVForm* m_pIVForm, long a_lParam=0)=0;
	///      
	virtual bool     STDMETHODCALLTYPE UnregisterIForm(IVForm* m_pIVForm)=0;

	///     
	virtual IVForm*  STDMETHODCALLTYPE GetIFormByName(const char* a_pchName)=0;
	//     
	virtual IVForm*  STDMETHODCALLTYPE GetIFormByIndex(ULONG a_nIndex)=0;


	///    
	virtual ITag*    STDMETHODCALLTYPE GetTagByName(const char* a_pchName)=0;
	///    
	virtual ITag*    STDMETHODCALLTYPE GetTagByIndex(ULONG a_nIndex)=0;
	///   
	virtual ULONG    STDMETHODCALLTYPE GetTagsCount()=0;



	///    
	virtual IModule* STDMETHODCALLTYPE GetModuleByIndex(ULONG a_nHostDeviceNum, ULONG a_nIndex)=0;
	///   
	virtual ULONG    STDMETHODCALLTYPE GetModulesCount(ULONG a_nHostDeviceNum)=0;

	///    
	virtual ITagsGroup* STDMETHODCALLTYPE GetGroupByIndex(ULONG a_nIndex) = 0;
	///    
	virtual ULONG       STDMETHODCALLTYPE GetGroupsCount() = 0;

	///   
	virtual RECORDERSTATE STDMETHODCALLTYPE GetState(RECORDERSTATE a_rsMask = RS_BASESTATE) = 0;
	
	
	///   
	virtual bool          STDMETHODCALLTYPE CheckState(RECORDERSTATE a_rsState)=0;


	///      
	virtual LPCSTR  STDMETHODCALLTYPE GetSignalFrameName() const = 0;
	///      
	virtual LPCSTR  STDMETHODCALLTYPE GetSignalFolderName() const = 0;

	///   
	virtual ITag*   STDMETHODCALLTYPE CreateTag(const char* pchName, LINKSTATE a_ls = LS_VIRTUAL, void* a_pParams = 0) = 0;
	///  
	virtual bool    STDMETHODCALLTYPE  CloseTag(ITag* piTag) = 0;

	///    
	virtual bool    STDMETHODCALLTYPE Notify(DWORD dwCommand, DWORD dwParam = 0) = 0;

	///    
	///      
	virtual void    STDMETHODCALLTYPE SetLastError(DWORD dwErrorCode) = 0;
	
	///       IRecorder
	virtual DWORD   STDMETHODCALLTYPE GetLastError() = 0;

	///       IRecorder   
	virtual LPCSTR  STDMETHODCALLTYPE ConvertErrorCodeToString(DWORD a_dwErrorCode) = 0;

	///  
	virtual HRESULT STDMETHODCALLTYPE GetProperty(DWORD dwPropertyID, VARIANT& Value)=0;
	///  
	virtual HRESULT STDMETHODCALLTYPE SetProperty(DWORD dwPropertyID, VARIANT Value)=0;


	///  

	///     
	virtual bool    STDMETHODCALLTYPE SetEnvironmentCurDir(const char* a_pchDir)=0;
	///     
	virtual bool    STDMETHODCALLTYPE GetEnvironmentCurDir(char* a_pchDir, int a_nLength)=0;

	///        
	virtual bool    STDMETHODCALLTYPE GetEnvironmentVar(const char* a_pchVarID, VARIANT& a_varVal)=0;
	///       
	virtual bool    STDMETHODCALLTYPE SetEnvironmentVar(const char* a_pchVarID, VARIANT& a_varVal)=0;

	///       long
	virtual bool    STDMETHODCALLTYPE GetEnvironmentLong(const char* a_pchVarID, long &a_lVal)=0;
	///       long
	virtual bool    STDMETHODCALLTYPE SetEnvironmentLong(const char* a_pchVarID, long a_lVal)=0;

	///       double
	virtual bool    STDMETHODCALLTYPE GetEnvironmentDouble(const char* a_pchVarID, double &a_dblVal)=0;
	///       double 
	virtual bool    STDMETHODCALLTYPE SetEnvironmentDouble(const char* a_pchVarID, double a_dblVal)=0;

	///       char*
	virtual bool    STDMETHODCALLTYPE GetEnvironmentString(const char* a_pchVarID, char* a_pchVal, int* a_pnLength)=0;

	///       char*
	virtual bool    STDMETHODCALLTYPE SetEnvironmentString(const char* a_pchVarID, const char* a_pchlVal)=0;

	///       
	virtual bool    STDMETHODCALLTYPE LogMessage(const char* a_pchMessage) = 0;

	///       
	virtual HWND    STDMETHODCALLTYPE GetHWND() = 0;
	virtual ITag*   STDMETHODCALLTYPE GetCurrentTag() = 0;
	virtual HRESULT STDMETHODCALLTYPE SetCurrentTag(ULONG a_nIndex) = 0;

	///     
	/// v1.06.2+
	virtual int STDMETHODCALLTYPE GetTagIndexByPointer(ITag* a_pTag) = 0;

	///   
	/// v1.09
	virtual int         STDMETHODCALLTYPE GetDevicesCount() = 0;

	///     *.cfg
	/// v1.09.2.3
	virtual const char* STDMETHODCALLTYPE GetProjectName() = 0;

	///   
	/// v1.10
	virtual const char* STDMETHODCALLTYPE GetRcBasePath() = 0;

	///   
	///    
	/// v1.10.1
	virtual ITagsGroup* STDMETHODCALLTYPE GetGroupByName(char* a_pchName) = 0;
	///  
	virtual HRESULT     STDMETHODCALLTYPE DeleteTagsGroup(ITagsGroup* a_pGrp) = 0;
	///  
	virtual ITagsGroup* STDMETHODCALLTYPE CreateTagsGroup(char* a_pchName) = 0;


	///  
	virtual HRESULT     STDMETHODCALLTYPE ImportSettings(const char* a_pchName, DWORD a_dwFlags)=0;
	///  
	virtual HRESULT     STDMETHODCALLTYPE ExportSettings(const char* a_pchName, DWORD a_dwFlags)=0;


	///      
	///      

	///    
	virtual HRESULT     STDMETHODCALLTYPE EnterConfigMode(IUnknown* pInitiator, DWORD dwFlags) = 0;

	///    
	virtual HRESULT     STDMETHODCALLTYPE LeaveConfigMode(IUnknown* pInitiator, DWORD dwFlags) = 0;

	///     
	virtual ULONG       STDMETHODCALLTYPE GetConfigModeState(DWORD& dwFlags) = 0;

	///   
	virtual IUnknown*   STDMETHODCALLTYPE GetConfigModeInitiator() = 0;



	///      

	///    
	virtual ULONG       STDMETHODCALLTYPE dvchGetAvailableChansCount()=0;
	///      
	virtual HRESULT     STDMETHODCALLTYPE dvchGetChansList(HARDLINK* a_pIDList)=0;
	///      
	virtual HRESULT     STDMETHODCALLTYPE dvchGetAddress(HARDLINK a_hlID, char* a_pchAddress, int* p_nBuffLen )=0;
	///        
	virtual HRESULT     STDMETHODCALLTYPE dvchGetDeviceName(HARDLINK a_hlID, char* a_pchDeviceName, int* p_nBuffLen)=0;
	///        
	virtual HRESULT     STDMETHODCALLTYPE dvchGetDeviceInfo(HARDLINK a_hlID, char* a_pchDeviceInfo, int* p_nBuffLen)=0;

	virtual HRESULT     STDMETHODCALLTYPE MultiTagSynchroReadDataBlock(
				DWORD dwCount, 
				ITag** pTag, 
				DWORD dwPortionLen, 
				double* pdblFreq, 
				double** ppdblData, 
				BOOL boolViaTransformer = TRUE
			) = 0;

	/// since v 1.19


	///        (bin)
	virtual const char* STDMETHODCALLTYPE MakeFullPath(const char* pchLocalPath) = 0;

	///    ,    
	virtual void STDMETHODCALLTYPE PushState(RECORDERSTATE a_rsState) = 0;

	///    ,    
	virtual void STDMETHODCALLTYPE PopState(RECORDERSTATE a_rsState) = 0;

	///      
	virtual HRESULT  STDMETHODCALLTYPE GetDeviceByIndex(PIDevice* a_pDevice, ULONG a_ulIndex) = 0;

	///      
	virtual ULONG    STDMETHODCALLTYPE GetDeviceInitStateByIndex(ULONG a_ulIndex) = 0;



	///     
	virtual HRESULT  STDMETHODCALLTYPE AddDevice(IDevice* a_pDevice, ULONG a_ulIndex=-1) = 0;
	///     
	virtual HRESULT  STDMETHODCALLTYPE RemoveDevice(ULONG a_ulIndex) = 0;

	virtual HRESULT STDMETHODCALLTYPE GetDeviceControlClassObject(void** ppv) = 0;
	virtual HRESULT STDMETHODCALLTYPE GetPluginsControlClassObject(void** ppv) = 0;


	virtual HRESULT STDMETHODCALLTYPE CreateInfoFile(const char *a_pchPath, ITagsGroup* a_pGroup)=0;

	///      
	virtual HRESULT STDMETHODCALLTYPE SetSignalFrameName(const char* a_pcFrameName)=0;



	///        /
	virtual double  STDMETHODCALLTYPE GetTimeCounter()=0;

	///     , 
	///     
	virtual ULONG   STDMETHODCALLTYPE GetSummaryReceivedPacketsCount()=0;


	///         ResetReceivedPacketsCount
	///     
	virtual ULONG   STDMETHODCALLTYPE GetReceivedPacketsCount()=0;
	virtual ULONG   STDMETHODCALLTYPE GetSummaryCounter()=0;
	virtual ULONG   STDMETHODCALLTYPE ResetReceivedPacketsCount()=0;

	virtual ULONG   STDMETHODCALLTYPE IncSummaryTime(LONGLONG a_liVal)=0;



	virtual HRESULT STDMETHODCALLTYPE GetWorkTime(double* a_dblTime)=0;    

	virtual HRESULT STDMETHODCALLTYPE ResetWorkTime()=0;


	///        IRQ
	virtual HRESULT STDMETHODCALLTYPE GetPerformanceTimeIRQ(double* a_dblTime) = 0;

	///         IRQ
	virtual HRESULT STDMETHODCALLTYPE ResetPerformanceTimeIRQ() = 0;

	///        
	virtual HRESULT STDMETHODCALLTYPE GetPerformanceTimeDriver(double* dblTime) = 0;

	///         
	virtual HRESULT STDMETHODCALLTYPE ResetPerformanceTimeDriver() = 0;

	///    
	virtual HRESULT STDMETHODCALLTYPE GetCPUUsage(double* pdblCPU) = 0;

	///     
	virtual HRESULT STDMETHODCALLTYPE dvchGetDeviceSerialNumber(
			HARDLINK hlID, 
			char* pchDeviceSerialNumber, 
			int* pnBuffLen
		) = 0;

	/// since v 2.7.0.9
	///      
	///      
	virtual HRESULT     STDMETHODCALLTYPE GetDeviceLastActivityTime(ULONG a_ulIndex, double* a_pdblTime)=0;

	/// since v 2.7.0.15
	///     
	virtual HRESULT     STDMETHODCALLTYPE dvchGetChanInfo(HARDLINK a_hlID, char* a_pchChanInfo, int* p_nBuffLen)=0;

	/// since v 2.7.1.0
	/// /     
	virtual HRESULT     STDMETHODCALLTYPE EnableCommonStartForDevice(IDevice* a_pDevice, VARIANT_BOOL a_vbEnable)=0;

	/// /         
	virtual HRESULT     STDMETHODCALLTYPE EnableCommonStopForDevice(IDevice* a_pDevice, VARIANT_BOOL a_vbEnable)=0;

	/// since v 2.7.5.0
	///       /   
	virtual double  STDMETHODCALLTYPE GetUTSTimeCounter() = 0;


	
	///      
	virtual HRESULT			STDMETHODCALLTYPE RunFn(void* a_fn,void* a_param) = 0;


	///   
	///      
	virtual HRESULT			STDMETHODCALLTYPE Destroy() = 0;

	///       
	virtual HRESULT			STDMETHODCALLTYPE GetPropertyAccess(IPropertyAccess** ppAccess, HANDLE h)=0;

	///        (data) sd:/Mera Files/Recorder
	virtual const char* STDMETHODCALLTYPE MakeFullPathData(const char* a_pchLocalPath)=0;

	/// since v 3.0.6.16
	///     
	virtual HRESULT     STDMETHODCALLTYPE dvchGetChanProperty(HARDLINK a_hlID, ULONG a_nPropID, VARIANT& a_varProp) = 0;
	
	/// since v 3.0.7.71
	///    TAGID
	virtual HRESULT     STDMETHODCALLTYPE GetTagByTagId(TAGID id, ITag** ppTag) = 0;
	

};



#endif
