#ifndef RCMACRO_H
#define RCMACRO_H
//#include "types.h"
#include <vector>
#include <string>

#include "uts.h"
#include "chkmask.h"

#define _CRT_SECURE_NO_WARNINGS

#define CHK_DELETE(pointer) if((pointer)!=NULL){delete (pointer); pointer=NULL;}

#define VALIDHANDLE(handle) ((((handle)==NULL)||((handle)==INVALID_HANDLE_VALUE))?FALSE:TRUE)

#define SAFECALL(object,function)VALIDHANDLE(object)?object->function:NULL;

#define VIRTUALTAGHANDLE ((HANDLE)0xFFFFFFFF)
#define VIRTUALTAGHANDLE_ ((CChannel*)0xFFFFFFFF)

#define VIRTUALTAGCHANID ((DWORD)0xFFFFFFFF)

typedef std::vector<std::string> strings;

//#define VOIDPPA (void**)&
//#define _VOIDPPA_(ptr) (void**)&(ptr)

inline double buf_as(void*p, VARTYPE vt, int i)
{
	switch(vt){
		case VT_R4:
			return reinterpret_cast<float*>(p)[i];
		case VT_I2:
			return reinterpret_cast<short*>(p)[i];
		case VT_UI2:
			return reinterpret_cast<USHORT*>(p)[i];
		case VT_I4:
			return reinterpret_cast<long*>(p)[i];
		case VT_UI4:
			return reinterpret_cast<ULONG*>(p)[i];
		case VT_R8:
			return reinterpret_cast<double*>(p)[i];
		default:
			throw "Unsupported type in buf_as";
	}
}

template <class T> 
inline void pair_put_in_bin(BYTE*p, T val)
{
	*((T*)(p+sizeof(double)))=val;
}


inline void pair_put_in_bin_as(BYTE*p, VARTYPE vt, double x, double val)
{
	((double*)p)[0]=x;
	switch(vt&~VT_PAIR){
		case VT_R8:
			pair_put_in_bin(p,val);
		break;
		case VT_R4:
			pair_put_in_bin(p,(float)val);
		break;
		case VT_UI2:
			pair_put_in_bin(p,(USHORT)val);
		break;
		case VT_I2:
			pair_put_in_bin(p,(SHORT)val);
		break;
	}
}


inline WORD SizeOfVarType(VARTYPE vtValue)
{
	switch(vtValue){
		case VT_I2:
			return sizeof(short);
		case VT_UI2:
			return sizeof(unsigned short);
		case VT_I4:
			return sizeof(long);
		case VT_UI4:
			return sizeof(unsigned);
		case VT_R4:
			return sizeof(float);
		case VT_R8:
			return sizeof(double);

		case VT_R8|VT_PAIR:
		case VT_R8|VT_UTS:
			return sizeof(double) * 2;
		case VT_UI2|VT_PAIR:
		case VT_I2|VT_PAIR:
			return sizeof(double) + sizeof(unsigned short);

		case VT_R4|VT_PAIR:
			return sizeof(double) + sizeof(float);

		case VT_UI4|VT_PAIR:
		case VT_I4|VT_PAIR:
			return sizeof(double) + sizeof(long);

		default:
			return sizeof(short);
	}
}



//    :
//    GetString((pBuff), (pLen))
//    0  in (pBuff)=NULL (pLen)=NULL  out  ERROR
//    1. in (pBuff)=buf  (pLen)=NULL  out  (pBuff)<-string 
//    2. in (pBuff)=NULL (pLen)=&len  out  *(pLen)=string_len
//    3. in (pBuff)=buf  (pLen)=&0    out  *(pLen)=string_len
//    4. in (pBuff)=buf  (pLen)=&len  
//       out  if *(pLen)>string_len then (pBuff)<-string  *(pLen)=string_len 
//            else ERROR


inline HRESULT SAFE_COPY_STRING(LPTSTR pBuff, int* pLen, LPCTSTR pString)
{
	HRESULT hr = S_OK;
	if(pBuff == NULL && pLen ==NULL) {
		hr = E_INVALIDARG;
	}
	else if(pBuff != NULL && pLen==NULL) {
		strcpy(pBuff, pString);
	}
	else if(pBuff == NULL && pLen != NULL) {
		*pLen = strlen(pString) + 1;
	}
	else if(pLen != NULL) {
		if(*pLen == 0) {
			*pLen = strlen(pString) + 1;
		}
		else if(pBuff != NULL && pLen != NULL) {
			if(*pLen <= (int)strlen(pString)) {
				*pLen = strlen(pString) + 1;
				hr = HRESULT_FROM_WIN32(ERROR_MORE_DATA);
			}
			else {
				strcpy(pBuff, pString);
				*pLen = strlen(pString) + 1;
			}
		}
	}
	return hr;
};

/*
#define SAFETY_COPY_STRING(pBuff, pLen, pString){\
 if((pBuff)==NULL && (pLen)==NULL)\
		return E_INVALIDARG;\
	if((pBuff)!=NULL && (pLen)==NULL){\
		strcpy((pBuff),(pString));\
		return S_OK;\
	}\
	if((pBuff)==NULL && (pLen)!=NULL){\
		*(pLen)=strlen((pString))+1;\
		return S_OK;\
	}\
	if((pLen)!=NULL)if(*(pLen)==0){\
		*(pLen)=strlen((pString))+1;\
		return S_OK;\
	}\
	if((pBuff)!=NULL && (pLen)!=NULL){\
		if(*(pLen)<=strlen((pString))){\
			*(pLen)=strlen((pString))+1;\
			return HRESULT_FROM_WIN32(ERROR_MORE_DATA);\
		}\
		strcpy((pBuff),(pString));\
		*(pLen)=strlen((pString))+1;\
		return S_OK;\
	}\
}
*/

#endif